@@ -40,6 +40,7 @@ enum {
 	COLUMN_MARKUP,
 	COLUMN_ICON,
 	COLUMN_ANNOT_MAPPING,
+	COLUMN_TOOLTIP,
 	N_COLUMNS
 };
 
@@ -107,7 +108,8 @@ ev_sidebar_annotations_create_simple_model (const gchar *message)
 	retval = (GtkTreeModel *)gtk_list_store_new (N_COLUMNS,
 						     G_TYPE_STRING,
 						     GDK_TYPE_PIXBUF,
-						     G_TYPE_POINTER);
+						     G_TYPE_POINTER,
+						     G_TYPE_STRING);
 
 	gtk_list_store_append (GTK_LIST_STORE (retval), &iter);
 	markup = g_strdup_printf ("<span size=\"larger\" style=\"italic\">%s</span>",
@@ -159,6 +161,9 @@ ev_sidebar_annotations_init (EvSidebarAnnotations *ev_annots)
 	gtk_tree_view_append_column (GTK_TREE_VIEW (ev_annots->priv->tree_view),
 				     column);
 
+	gtk_tree_view_set_tooltip_column (GTK_TREE_VIEW (ev_annots->priv->tree_view),
+					  COLUMN_TOOLTIP);
+
 	gtk_container_add (GTK_CONTAINER (ev_annots->priv->swindow), ev_annots->priv->tree_view);
 	gtk_widget_show (ev_annots->priv->tree_view);
 
@@ -228,6 +233,13 @@ ev_sidebar_annotations_annot_added (EvSidebarAnnotations *sidebar_annots,
 	ev_sidebar_annotations_load (sidebar_annots);
 }
 
+void
+ev_sidebar_annotations_annot_changed (EvSidebarAnnotations *sidebar_annots,
+				      EvAnnotation         *annot)
+{
+	ev_sidebar_annotations_load (sidebar_annots);
+}
+
 void
 ev_sidebar_annotations_annot_removed (EvSidebarAnnotations *sidebar_annots)
 {
@@ -383,7 +395,8 @@ job_finished_callback (EvJobAnnots          *job,
 	model = gtk_tree_store_new (N_COLUMNS,
 				    G_TYPE_STRING,
 				    GDK_TYPE_PIXBUF,
-				    G_TYPE_POINTER);
+				    G_TYPE_POINTER,
+				    G_TYPE_STRING);
 
 	screen = gtk_widget_get_screen (GTK_WIDGET (sidebar_annots));
 	icon_theme = gtk_icon_theme_get_for_screen (screen);
@@ -408,7 +421,9 @@ job_finished_callback (EvJobAnnots          *job,
 			EvAnnotation *annot;
 			const gchar  *label;
 			const gchar  *modified;
-			gchar        *markup;
+			const gchar  *contents;
+			gchar        *markup = NULL;
+			gchar        *tooltip = NULL;
 			GtkTreeIter   child_iter;
 			GdkPixbuf    *pixbuf = NULL;
 
@@ -418,12 +433,18 @@ job_finished_callback (EvJobAnnots          *job,
 
 			label = ev_annotation_markup_get_label (EV_ANNOTATION_MARKUP (annot));
 			modified = ev_annotation_get_modified (annot);
-			if (modified) {
-				markup = g_strdup_printf ("<span weight=\"bold\">%s</span>\n%s",
-							  label, modified);
-			} else {
-				markup = g_strdup_printf ("<span weight=\"bold\">%s</span>", label);
-			}
+			contents = ev_annotation_get_contents (annot);
+
+			if (modified)
+				tooltip = g_strdup_printf ("<span weight=\"bold\">%s</span>\n%s",
+							   label, modified);
+			else
+				tooltip = g_strdup_printf ("<span weight=\"bold\">%s</span>", label);
+
+			if (contents)
+				markup = g_strdup_printf ("%s", contents);
+			else
+				markup = g_strdup_printf ("%s", tooltip);
 
 			if (EV_IS_ANNOTATION_TEXT (annot)) {
 				if (!text_icon) {
@@ -489,8 +510,10 @@ job_finished_callback (EvJobAnnots          *job,
 					    COLUMN_MARKUP, markup,
 					    COLUMN_ICON, pixbuf,
 					    COLUMN_ANNOT_MAPPING, ll->data,
+					    COLUMN_TOOLTIP, tooltip,
 					    -1);
 			g_free (markup);
+			g_free (tooltip);
 			found = TRUE;
 		}
 
@@ -500,6 +523,7 @@ job_finished_callback (EvJobAnnots          *job,
 
 	gtk_tree_view_set_model (GTK_TREE_VIEW (priv->tree_view),
 				 GTK_TREE_MODEL (model));
+	gtk_tree_view_expand_all (GTK_TREE_VIEW (priv->tree_view));
 	g_object_unref (model);
 
 	if (text_icon)
